#!/usr/bin/perl -w

=head1 NAME

Debconf::Element::Noninteractive::Boolean - dummy boolean Element

=cut

package Debconf::Element::Noninteractive::Boolean;
use strict;
use base qw(Debconf::Element::Noninteractive);

=head1 DESCRIPTION

This is a dummy boolean element.

=cut

1
